﻿using UnityEngine;
using System;

namespace lockemx
{
	public class LieStill : MVRScript
	{
		public JSONStorableFloat TiltDegreesJson;

		protected float TiltDegreesLast;

		protected float ModifyDelaySpent = 0.0f;
		protected bool ModifyDone = true;

		protected GameObject CameraEye;
		protected GameObject HeightOffset;
		protected Quaternion SourceRot;
		protected Quaternion TargetRot;
		protected float TimeCount = 1.1f;
		
		private string HeightOffsetName = "HeightOffset";

		public override void Init()
		{
			base.Init();

			TiltDegreesJson = new JSONStorableFloat("Tilt Degrees", 0.0f, -180.0f, 180.0f, true)
			{
				storeType = JSONStorableParam.StoreType.Full
			};
			TiltDegreesLast = TiltDegreesJson.val;
			RegisterFloat(TiltDegreesJson);
			CreateSlider(TiltDegreesJson, true);

			CameraEye = (SuperController.singleton.isOpenVR
				? SuperController.singleton.ViveCenterCamera
				: SuperController.singleton.OVRCenterCamera).gameObject;
			HeightOffset = CameraEye.transform.parent.parent.gameObject;
			if (HeightOffset.name != HeightOffsetName) {
				for (var i = 0; i < 5; i++) {
					HeightOffset = HeightOffset.transform.parent.gameObject;
					if (HeightOffset.name == HeightOffsetName) break;
				}

				if (HeightOffset.name != HeightOffsetName) {
					throw new ApplicationException(String.Format("{0}: Only Found CameraEye: {1} and HeightOffset: {2}. Disabling.", nameof(LieStill), CameraEye.name, HeightOffset.name));
				}
			}
			SuperController.LogMessage(String.Format("{0}: Found CameraEye: {1} and HeightOffset: {2}", nameof(LieStill), CameraEye.name, HeightOffset.name));
		}

		protected static Quaternion RotationUp(Transform t)
		{
			return Quaternion.FromToRotation(t.up, Vector3.up) * t.rotation;
		}

		private void PrepareRot(Transform t, Quaternion tgt)
		{
			SourceRot = t.rotation;
			TargetRot = tgt;
			TimeCount = 0.0f;
		}

		protected void OnDisable()
		{
			var t = HeightOffset.transform;
			t.rotation = RotationUp(t);
		}

        protected void OnEnable()
		{
			var t = HeightOffset.transform;
			PrepareRot(t, Quaternion.AngleAxis(TiltDegreesJson.val, t.right) * RotationUp(t));
		}

		protected void LateUpdate()
		{
			var t = HeightOffset.transform;

			if (Math.Abs(TiltDegreesLast - TiltDegreesJson.val) > 0.001f)
			{
				TiltDegreesLast = TiltDegreesJson.val;
				ModifyDelaySpent = 0.0f;
				ModifyDone = false;
			}

			if (ModifyDelaySpent < 0.05f)
			{
				ModifyDelaySpent += Time.deltaTime;
			}
			else
			{
				if (!ModifyDone)
				{
					PrepareRot(t, Quaternion.AngleAxis(TiltDegreesJson.val, t.right) * RotationUp(t));
					ModifyDone = true;
				}
			}

			if (!(TimeCount <= 1.0f)) return;
			t.rotation = Quaternion.Slerp(SourceRot, TargetRot, TimeCount);
			TimeCount += Time.deltaTime * 1.5f;
		}

		private void OnDestroy()
		{
			var t = HeightOffset.transform;
			t.rotation = RotationUp(t);
		}
	}
}
